/*
 * Copyright (c) 2011 Kevin Smith
 * Licensed under the GNU General Public License v3.
 * See Documentation/Licenses/GPLv3.txt for more information.
 */


#include "Roster/QtOccupantListWidget.h"

#include <QContextMenuEvent>
#include <QMenu>
#include <QAction>
#include <QInputDialog>

#include "Swift/Controllers/Roster/ContactRosterItem.h"
#include "Swift/Controllers/Roster/GroupRosterItem.h"
#include "Swift/Controllers/UIEvents/UIEventStream.h"
#include "QtSwiftUtil.h"

namespace Swift {

QtOccupantListWidget::QtOccupantListWidget(UIEventStream* eventStream, QWidget* parent) : QtTreeWidget(eventStream, parent) {

}

QtOccupantListWidget::~QtOccupantListWidget() {

}

void QtOccupantListWidget::setAvailableOccupantActions(const std::vector<ChatWindow::OccupantAction>& actions) {
	availableOccupantActions_ = actions;
}

void QtOccupantListWidget::contextMenuEvent(QContextMenuEvent* event) {
	QModelIndex index = indexAt(event->pos());
	if (!index.isValid()) {
		return;
	}
	RosterItem* item = static_cast<RosterItem*>(index.internalPointer());
	ContactRosterItem* contact = dynamic_cast<ContactRosterItem*>(item);
	if (contact) {
		QMenu contextMenu;
		std::map<QAction*, ChatWindow::OccupantAction> actions;
		foreach (ChatWindow::OccupantAction availableAction, availableOccupantActions_) {
			QString text = "Error: missing string";
			switch (availableAction) {
				case ChatWindow::Kick: text = tr("Kick user"); break;
			}
			QAction* action = contextMenu.addAction(text);
			actions[action] = availableAction;
		}
		QAction* result = contextMenu.exec(event->globalPos());
		if (result) {
			onOccupantActionSelected(actions[result], contact);
		}
	}
}

}

